#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <conio.h>

char    Motorzustand='\x00';		// global variable
HANDLE	hCom;						// handle to COM device  

// define all functions

char clearBit(char wert, int bit);
char setBit(char wert,int bit);
BOOL testBit(char wert ,int bit);
void motor(int num, char dir);
void Fischer_install();
char input_word();
BOOL switch_in(char num );
void output_word(char word );
void motors_off();
int  analog_in(char num );

// example for using the functions

void main()
{ 
	
	Fischer_install();				// initialize the fischer interface
	motor(3,'R');					// turn on motor 3 right
	do {} while (!switch_in(4));	// wait for E4
	motor(3,'A');					// turn off motor 3

	motor(1,'R');
	do {} while (!switch_in(3));
	motor(1,'A');	

	motor(2,'R');
	do {} while (!switch_in(2));
	motor(2,'A');

	motor(1,'L');
	do {} while (!switch_in(7));
	motor(1,'A');	

	motor(2,'L');
	do {} while (!switch_in(6));
	motor(2,'A');

	motor(1,'R');
	do {} while (!switch_in(3));
	motor(1,'A');	

	motor(2,'R');
	do {} while (!switch_in(2));
	motor(2,'A');
}


/* Fischer_install()
	
	sets the serial parameters to
	9600 Baud
	no parity
	8 Bit
	1 stopbit

	open the serial connection

	This program uses serial port 2. If you need another port
	please change the parameter COM2 to COMx. This function also
	turns all motors off.

*/


void Fischer_install()

{

	DCB dcb;
	DWORD dwError;
	BOOL fSuccess;

	hCom = CreateFile( 
    "COM2",							/* name of COM device to open */ 
    GENERIC_READ | GENERIC_WRITE,	/* read-write access */ 
    0,								/* not used                  */ 
    NULL,							/* not used                  */ 
    OPEN_EXISTING,					/* required for tape devices */ 
    0,								/* not used                  */ 
    NULL);							/* not used                  */ 
    
	
	if (hCom == INVALID_HANDLE_VALUE)
	{
		dwError = GetLastError();
	}

	fSuccess = GetCommState(hCom, &dcb);

	if (!fSuccess) 
	{
    /* Handle the error. */
	}

	/* Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. */

	dcb.BaudRate = 9600;			// 9600 Baud  
	dcb.ByteSize = 8;				// 8 Bits
	dcb.Parity = NOPARITY;			// no parity
	dcb.StopBits = ONESTOPBIT;		// 1 stopbit

	fSuccess = SetCommState(hCom, &dcb);

	if (!fSuccess) 
	{
    /* Handle the error. */
	}
    motors_off();					// motors off
} 
  
/*  input_word()
	gets a full word representing all inputs (1..8)
*/


char input_word()
{
	LPSTR	Befehl="  "; 
	LPSTR	Zustand=" "; 
	BOOL	Result; 
	DWORD	Written; 
	DWORD	Read; 

	Befehl[0]='\xc1';												// 1. Byte : 193 in Hex
	Befehl[1]=Motorzustand;											// 2. Byte : Motorzustand


  	Result = WriteFile(hCom,Befehl,2,&Written,NULL);				// send 2 Byte command & motors
  	Result = ReadFile(hCom,Zustand,1,&Read,NULL);					// read 1 Byte interface status
	
	return Zustand[0];

}

/*  motor(num,dir)
	operates the designated motor ( num =1 to 8 )
	according to dir (A,L,R).
	A = off
	L = left
	R = right
*/

void motor(int num, char dir)
{    
	char	Result; 
	switch (dir)													// switch direction
	{
	case 'A':														// off
			{
			Motorzustand = clearBit(Motorzustand, (num * 2));		// clear right bit
			Motorzustand = clearBit(Motorzustand, (num * 2 - 1));	// clear left bit
			break;
			}
	case 'L':                                                       // left
			{
			Motorzustand = clearBit(Motorzustand, (num * 2));		// set right bit
			Motorzustand = setBit(Motorzustand, (num * 2 - 1));		// clear left bit
			break;
			}
	case 'R':														// right
			{
			Motorzustand = setBit(Motorzustand, (num * 2));			// set right bit
			Motorzustand = clearBit(Motorzustand, (num * 2 - 1));	// clear left bit
			break;
			}
	}

	Result = input_word();											// write motorbyte to the interface

}
/*  clearBit(wert,bit)
	clear bit 'bit' in Byte 'wert'
*/

 
char clearBit(char wert, int bit)
{
	int bitvorne;
	int bithinten;
    div_t vorne;
    div_t hinten;

	bitvorne = int(pow(2,bit));			
	vorne = div(wert,bitvorne);
	bithinten = int(pow(2,(bit-1)));
	hinten = div(wert,bithinten);
	
	return (vorne.quot * bitvorne + hinten.rem);
}
/*  setBit(wert,bit)
	set bit 'bit' in Byte 'wert'
*/
char setBit(char wert,int bit)
{
	int bitvorne;
	int bithinten;
    int vorne_neu;
	div_t vorne;
    div_t hinten;

	bitvorne = int(pow(2,bit));
	vorne = div(wert,bitvorne);
	vorne_neu=vorne.quot*2+1;
	bithinten = int(pow(2,(bit-1)));
	hinten = div(wert,bithinten);
	
	return (vorne_neu * bithinten + hinten.rem);
}
/*  testBit(wert,bit)
	which value has bit 'bit' in Byte 'wert'
*/
BOOL testBit(char wert ,int bit)
{
	int bitvorne;
	int bithinten;
    int vorne_neu;
	char temp;
	div_t vorne;
    div_t hinten;

	bitvorne = int(pow(2,bit));
	vorne = div(wert,bitvorne);
	vorne_neu=vorne.quot*bitvorne;
	bithinten = int(pow(2,(bit-1)));
	hinten = div(wert,bithinten);
	temp=(wert-vorne_neu - hinten.rem);
	
	return (temp);
}

/*  switch_in(num)
	get the current state of teh designated
	switch input ( num = 1 to 8 ). If the input
	is off, 0 is returned. If on, 1 is returned.
*/

BOOL switch_in(char num )
{
	char result;

	result = input_word();			// read the input status
	
	return testBit(result, num);	// which value has the interesting bit
}

/*  motors_off()
	turn off all motors
*/

void motors_off()
{
	char result;

	Motorzustand = 0;				// reset all
	result = input_word();			// send motor settings to interface
}

/*  output_word(word)
	outputs a full word that controls all motor outputs 1 to 8 at the same time
*/

void output_word(char word )
{
	char result;

	Motorzustand = word;				// set motor settings
	result = input_word();				// send motor settings to interface
}

/*  analog_in(num)
	gets the current analog input value of EX or EY ( num = 0 or 1 ).
*/

int analog_in(char num )
{
	LPSTR	Befehl="  "; 
	LPSTR	Zustand="   "; 
	BOOL	Result; 
	DWORD	Written; 
	DWORD	Read; 
	
	switch (num)											// switch num
	{
	case 0:	Befehl[0]='\xc5';								// 1. Byte : 197 in Hex for EX
	case 1:	Befehl[0]='\xc9';								// 1. Byte : 201 in Hex for EY
	}
	Befehl[1]=Motorzustand;									// 2. Byte : motor setting
 	
	Result = WriteFile(hCom,Befehl,2,&Written,NULL);		// send 2 Byte
  	Result = ReadFile(hCom,Zustand,3,&Read,NULL);			// read 3 Byte
															// 1. Byte is the status of Inputs E1 to E8
															// 2. Byte and 3 Byte is the value of the analog input

	return((Zustand[1] * 255) + Zustand[2]);				// return integer value
}
